import com.silanis.esl.sdk.external.signer.verification.domain.request.SignerVerificationRequest;
import com.silanis.esl.sdk.external.signer.verification.domain.request.SignerVerificationRequestPayload;
import com.silanis.esl.sdk.external.signer.verification.domain.signer.SignerInfo;
import com.silanis.esl.sdk.external.signer.verification.domain.transaction.TransactionInfo;
import com.silanis.esl.sdk.external.signer.verification.processor.DefaultSignerVerificationProcessorFactory;
import com.silanis.esl.sdk.external.signer.verification.processor.SignerVerificationConfig;
import com.silanis.esl.sdk.external.signer.verification.processor.SignerVerificationProcessor;
import com.silanis.esl.sdk.external.signer.verification.processor.SignerVerificationProcessorFactory;
import com.silanis.esl.sdk.external.signer.verification.processor.jwt.HmacJwtSignerVerificationConfig;

import java.net.URISyntaxException;
import java.util.UUID;

public class VerificationRequestTokenConversion {
    private static final String SIGNING_SHARED_SECRET = "signing secret";
    private static final String ENCRYPTION_SHARED_SECRET = "encryption secret";
    private static final SignerVerificationProcessor signerVerificationProcessor;

    static {
        SignerVerificationConfig signerVerificationConfig = new HmacJwtSignerVerificationConfig(SIGNING_SHARED_SECRET, ENCRYPTION_SHARED_SECRET);
        SignerVerificationProcessorFactory signerVerificationProcessorFactory = new DefaultSignerVerificationProcessorFactory();
        signerVerificationProcessor = signerVerificationProcessorFactory.createSignerVerificationProcessor(signerVerificationConfig);
    }

    public static void main(String[] args) throws URISyntaxException {
        String signerVerificationRequestToken = retrieveSignerVerificationRequestToken();
        SignerVerificationRequest signerVerificationRequest = signerVerificationProcessor.toSignerVerificationRequest(new SignerVerificationRequestPayload(signerVerificationRequestToken));
        TransactionInfo transactionInfo = signerVerificationRequest.getTransactionInfo();
        SignerInfo signerInfo = signerVerificationRequest.getSignerInfo();

        String userIdentityInfo = signerVerificationRequest.getUserIdentityInfo();  //The actual Verification Server will have to parse it to determine which user to verify

        String callbackUrl = signerVerificationRequest.getCallbackUrl();    //Root URL for redirecting the signer back to eSignLive
        String documentIdentificationInfo = signerVerificationRequest.getDocumentIdentificationInfo();  //To be returned as part of the verification response later

        System.out.println("Transaction Name: " + transactionInfo.getTransactionName());
        System.out.println("Document Name: " + transactionInfo.getDocumentName());
        System.out.println("Signer: " + signerInfo.getSignerFirstName() + " " + signerInfo.getSignerLastName() + " - " + signerInfo.getSignerEmail());
    }

    /**
        For the actual implementation of the Verification Server, this should extract the Signer Verification Request Token from the redirect URL.
        Here, we simply generate a sample token so that we can show how to convert it to a SignerVerificationRequest object in the main method above.
    */
    private static String retrieveSignerVerificationRequestToken() {
        SignerVerificationRequest signerVerificationRequest = new SignerVerificationRequest()
                .setCallbackUrl("https://apps.esignlive.com/transaction/" + UUID.randomUUID().toString() + "/documents/" + UUID.randomUUID().toString())
                .setDocumentIdentificationInfo(UUID.randomUUID().toString())
                .setUserIdentityInfo(UUID.randomUUID().toString())
                .setTransactionInfo(new TransactionInfo()
                        .setTransactionName("Mortgage Application")
                        .setDocumentName("Mortgage Application Form"))
                .setSignerInfo(new SignerInfo()
                        .setSignerEmail("peter.young@yahoo.ca")
                        .setSignerFirstName("Peter")
                        .setSignerLastName("Young"));
        return signerVerificationProcessor.toSignerVerificationRequestPayload(signerVerificationRequest).getValue();
    }
}
